@@ -9,6 +9,7 @@ from typing import Iterator
 from typing import Mapping
 from typing import Sequence
 
+from narwhals._namespace import is_native_spark_like
 from narwhals._spark_like.utils import evaluate_exprs
 from narwhals._spark_like.utils import import_functions
 from narwhals._spark_like.utils import import_native_dtypes
@@ -22,7 +23,6 @@ from narwhals.utils import check_column_exists
 from narwhals.utils import find_stacklevel
 from narwhals.utils import generate_temporary_column_name
 from narwhals.utils import import_dtypes_module
-from narwhals.utils import is_spark_like_dataframe
 from narwhals.utils import not_implemented
 from narwhals.utils import parse_columns_to_drop
 from narwhals.utils import parse_version
@@ -56,7 +56,7 @@ Incomplete: TypeAlias = Any  # pragma: no cover
 
 class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"]):
     def __init__(
-        self: Self,
+        self,
         native_dataframe: SQLFrameDataFrame,
         *,
         backend_version: tuple[int, ...],
@@ -68,10 +68,11 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
         self._implementation = implementation
         self._version = version
         self._cached_schema: dict[str, DType] | None = None
+        self._cached_columns: list[str] | None = None
         validate_backend_version(self._implementation, self._backend_version)
 
     @property
-    def _F(self: Self):  # type: ignore[no-untyped-def] # noqa: ANN202, N802
+    def _F(self):  # type: ignore[no-untyped-def] # noqa: ANN202, N802
         if TYPE_CHECKING:
             from sqlframe.base import functions
 
@@ -80,7 +81,7 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             return import_functions(self._implementation)
 
     @property
-    def _native_dtypes(self: Self):  # type: ignore[no-untyped-def] # noqa: ANN202
+    def _native_dtypes(self):  # type: ignore[no-untyped-def] # noqa: ANN202
         if TYPE_CHECKING:
             from sqlframe.base import types
 
@@ -89,7 +90,7 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             return import_native_dtypes(self._implementation)
 
     @property
-    def _Window(self: Self) -> type[Window]:  # noqa: N802
+    def _Window(self) -> type[Window]:  # noqa: N802
         if TYPE_CHECKING:
             from sqlframe.base.window import Window
 
@@ -99,7 +100,7 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
 
     @staticmethod
     def _is_native(obj: SQLFrameDataFrame | Any) -> TypeIs[SQLFrameDataFrame]:
-        return is_spark_like_dataframe(obj)
+        return is_native_spark_like(obj)
 
     @classmethod
     def from_native(cls, data: SQLFrameDataFrame, /, *, context: _FullContext) -> Self:
@@ -110,10 +111,10 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             implementation=context._implementation,
         )
 
-    def __native_namespace__(self: Self) -> ModuleType:  # pragma: no cover
+    def __native_namespace__(self) -> ModuleType:  # pragma: no cover
         return self._implementation.to_native_namespace()
 
-    def __narwhals_namespace__(self: Self) -> SparkLikeNamespace:
+    def __narwhals_namespace__(self) -> SparkLikeNamespace:
         from narwhals._spark_like.namespace import SparkLikeNamespace
 
         return SparkLikeNamespace(
@@ -122,10 +123,10 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             implementation=self._implementation,
         )
 
-    def __narwhals_lazyframe__(self: Self) -> Self:
+    def __narwhals_lazyframe__(self) -> Self:
         return self
 
-    def _with_version(self: Self, version: Version) -> Self:
+    def _with_version(self, version: Version) -> Self:
         return self.__class__(
             self.native,
             backend_version=self._backend_version,
@@ -133,7 +134,7 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             implementation=self._implementation,
         )
 
-    def _with_native(self: Self, df: SQLFrameDataFrame) -> Self:
+    def _with_native(self, df: SQLFrameDataFrame) -> Self:
         return self.__class__(
             df,
             backend_version=self._backend_version,
@@ -186,11 +187,17 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             yield self._F.col(col)
 
     @property
-    def columns(self: Self) -> list[str]:
-        return list(self.schema)
+    def columns(self) -> list[str]:
+        if self._cached_columns is None:
+            self._cached_columns = (
+                list(self.schema)
+                if self._cached_schema is not None
+                else self.native.columns
+            )
+        return self._cached_columns
 
     def collect(
-        self: Self,
+        self,
         backend: ModuleType | Implementation | str | None,
         **kwargs: Any,
     ) -> CompliantDataFrame[Any, Any, Any]:
@@ -234,11 +241,11 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
         msg = f"Unsupported `backend` value: {backend}"  # pragma: no cover
         raise ValueError(msg)  # pragma: no cover
 
-    def simple_select(self: Self, *column_names: str) -> Self:
+    def simple_select(self, *column_names: str) -> Self:
         return self._with_native(self.native.select(*column_names))
 
     def aggregate(
-        self: Self,
+        self,
         *exprs: SparkLikeExpr,
     ) -> Self:
         new_columns = evaluate_exprs(self, *exprs)
@@ -247,25 +254,25 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
         return self._with_native(self.native.agg(*new_columns_list))
 
     def select(
-        self: Self,
+        self,
         *exprs: SparkLikeExpr,
     ) -> Self:
         new_columns = evaluate_exprs(self, *exprs)
         new_columns_list = [col.alias(col_name) for (col_name, col) in new_columns]
         return self._with_native(self.native.select(*new_columns_list))
 
-    def with_columns(self: Self, *exprs: SparkLikeExpr) -> Self:
+    def with_columns(self, *exprs: SparkLikeExpr) -> Self:
         new_columns = evaluate_exprs(self, *exprs)
         return self._with_native(self.native.withColumns(dict(new_columns)))
 
-    def filter(self: Self, predicate: SparkLikeExpr) -> Self:
+    def filter(self, predicate: SparkLikeExpr) -> Self:
         # `[0]` is safe as the predicate's expression only returns a single column
         condition = predicate._call(self)[0]
         spark_df = self.native.where(condition)
         return self._with_native(spark_df)
 
     @property
-    def schema(self: Self) -> dict[str, DType]:
+    def schema(self) -> dict[str, DType]:
         if self._cached_schema is None:
             self._cached_schema = {
                 field.name: native_to_narwhals_dtype(
@@ -277,25 +284,25 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             }
         return self._cached_schema
 
-    def collect_schema(self: Self) -> dict[str, DType]:
+    def collect_schema(self) -> dict[str, DType]:
         return self.schema
 
-    def drop(self: Self, columns: Sequence[str], *, strict: bool) -> Self:
+    def drop(self, columns: Sequence[str], *, strict: bool) -> Self:
         columns_to_drop = parse_columns_to_drop(
             compliant_frame=self, columns=columns, strict=strict
         )
         return self._with_native(self.native.drop(*columns_to_drop))
 
-    def head(self: Self, n: int) -> Self:
+    def head(self, n: int) -> Self:
         return self._with_native(self.native.limit(num=n))
 
-    def group_by(self: Self, *keys: str, drop_null_keys: bool) -> SparkLikeLazyGroupBy:
+    def group_by(self, *keys: str, drop_null_keys: bool) -> SparkLikeLazyGroupBy:
         from narwhals._spark_like.group_by import SparkLikeLazyGroupBy
 
         return SparkLikeLazyGroupBy(self, keys, drop_null_keys=drop_null_keys)
 
     def sort(
-        self: Self,
+        self,
         *by: str,
         descending: bool | Sequence[bool],
         nulls_last: bool,
@@ -317,11 +324,11 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
         sort_cols = [sort_f(col) for col, sort_f in zip(by, sort_funcs)]
         return self._with_native(self.native.sort(*sort_cols))
 
-    def drop_nulls(self: Self, subset: Sequence[str] | None) -> Self:
+    def drop_nulls(self, subset: Sequence[str] | None) -> Self:
         subset = list(subset) if subset else None
         return self._with_native(self.native.dropna(subset=subset))
 
-    def rename(self: Self, mapping: Mapping[str, str]) -> Self:
+    def rename(self, mapping: Mapping[str, str]) -> Self:
         rename_mapping = {
             colname: mapping.get(colname, colname) for colname in self.columns
         }
@@ -348,7 +355,7 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
         return self._with_native(self.native.dropDuplicates(subset=subset))
 
     def join(
-        self: Self,
+        self,
         other: Self,
         how: JoinStrategy,
         left_on: Sequence[str] | None,
@@ -384,7 +391,7 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
         # If how in {"semi", "anti"}, then resulting columns are same as left columns
         # Otherwise, we add the right columns with the new mapping, while keeping the
         # original order of right_columns.
-        col_order = left_columns
+        col_order = left_columns.copy()
 
         if how in {"inner", "left", "cross"}:
             col_order.extend(
@@ -410,12 +417,11 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             else left_on_
         )
         how_native = "full_outer" if how == "full" else how
-
         return self._with_native(
             self.native.join(other_native, on=on_, how=how_native).select(col_order)
         )
 
-    def explode(self: Self, columns: Sequence[str]) -> Self:
+    def explode(self, columns: Sequence[str]) -> Self:
         dtypes = import_dtypes_module(self._version)
 
         schema = self.collect_schema()
@@ -483,7 +489,7 @@ class SparkLikeLazyFrame(CompliantLazyFrame["SparkLikeExpr", "SQLFrameDataFrame"
             raise AssertionError(msg)
 
     def unpivot(
-        self: Self,
+        self,
         on: Sequence[str] | None,
         index: Sequence[str] | None,
         variable_name: str,
